@@ -15,7 +15,7 @@ consistency_check <- function(obj, case, ...) {
           if (!is.null(dim(obj))) {
                stop("The series must be univariate vectors (provided object has non-NULL dim)")
           }
-          if (length(obj) < 1) {
+          if (length(obj) < 1L) {
                stop("The series must have at least one point")
           }
           if (any(is.na(obj))) {
@@ -28,7 +28,7 @@ consistency_check <- function(obj, case, ...) {
           if (class(obj) != "list")
                stop("Series must be provided in a list")
 
-          if (length(obj) < 1)
+          if (length(obj) < 1L)
                stop("Data is empty")
 
           if (any(sapply(obj, function(obj) !is.null(dim(obj)) )))
@@ -37,10 +37,10 @@ consistency_check <- function(obj, case, ...) {
           if (any(!sapply(obj, is.numeric)))
                stop("Each element of the list must be a numeric vector")
 
-          if (length(unique(Lengths)) > 1)
+          if (length(unique(Lengths)) > 1L)
                stop("All series must have the same length")
 
-          if (any(Lengths < 1))
+          if (any(Lengths < 1L))
                stop("All series must have at least one element")
 
           if (any( sapply(obj, function(ts) {any(is.na(ts))}) ))
@@ -54,7 +54,7 @@ consistency_check <- function(obj, case, ...) {
           if (class(obj) != "list")
                stop("Series must be provided in a list")
 
-          if (length(obj) < 1)
+          if (length(obj) < 1L)
                stop("Data is empty")
 
           if (any(sapply(obj, function(obj) !is.null(dim(obj)) )))
@@ -63,7 +63,7 @@ consistency_check <- function(obj, case, ...) {
           if (any(!sapply(obj, is.numeric)))
                stop("Each element of the list must be a numeric vector")
 
-          if (any(Lengths < 1))
+          if (any(Lengths < 1L))
                stop("All series must have at least one element")
 
           if (any( sapply(obj, function(ts) {any(is.na(ts))}) ))
@@ -73,7 +73,7 @@ consistency_check <- function(obj, case, ...) {
           if (is.null(obj))
                stop("Please provide the 'window.size' parameter")
 
-          if (obj <= 0)
+          if (obj <= 0L)
                stop("Window width must be larger than 0")
 
           return(as.integer(obj))
@@ -82,7 +82,7 @@ consistency_check <- function(obj, case, ...) {
 
           if (is.matrix(obj)) {
                Names <- rownames(obj)
-               obj <- lapply(seq_len(nrow(obj)), function(i) obj[i,])
+               obj <- lapply(seq_len(nrow(obj)), function(i) obj[i, ])
                names(obj) <- Names
 
           } else if (is.numeric(obj)) {
@@ -97,7 +97,7 @@ consistency_check <- function(obj, case, ...) {
           return(obj)
 
      } else if (case == "dist") {
-          .local <- function(obj, trace = FALSE, lengths = FALSE, silent = TRUE, ...) {
+          .local <- function(obj, trace = FALSE, Lengths = FALSE, silent = TRUE, ...) {
                included <- c("dtw", "dtw2", "dtw_lb", "lbk", "lbi", "sbd")
                valid <- c("dtw", "dtw2", "sbd")
 
@@ -109,11 +109,11 @@ consistency_check <- function(obj, case, ...) {
                          stop("Please provide a valid distance function registered with the proxy package.")
                }
 
-               if (lengths) {
+               if (Lengths) {
                     if ((obj %in% included) && !(obj %in% valid))
-                         stop("Only the following distances are supported for series with different lengths:\n\tdtw\tdtw2\tsbd")
+                         stop("Only the following distances are supported for series with different length:\n\tdtw\tdtw2\tsbd")
                     else if(!(obj %in% included) && trace)
-                         message("Series have different lengths. Please confirm that the provided distance function supports this.")
+                         message("Series have different length. Please confirm that the provided distance function supports this.")
                }
 
                TRUE # valid registered distance
@@ -122,13 +122,13 @@ consistency_check <- function(obj, case, ...) {
           return(.local(obj, ...))
 
      } else if (case == "cent") {
-          included <- c("mean", "median", "shape", "dba", "pam")
+          included <- c("mean", "median", "shape", "dba", "pam", "fcm")
           valid <- c("dba", "pam", "shape")
 
           if (is.character(obj) && (obj %in% included) && !(obj %in% valid))
-               stop("Only the following centroids are supported for series with different lengths:\n\tdba\tpam\tshape")
+               stop("Only the following centroids are supported for series with different length:\n\tdba\tpam\tshape")
           else if(is.character(obj) && !(obj %in% included) && list(...)$trace)
-               message("Series have different lengths. Please confirm that the provided centroid function supports this.")
+               message("Series have different length. Please confirm that the provided centroid function supports this.")
 
      } else {
           stop("Possibly a typo in function consistency_check")
@@ -137,6 +137,26 @@ consistency_check <- function(obj, case, ...) {
      invisible(NULL)
 }
 
+# ========================================================================================================
+# Membership update for fuzzy c-means clustering
+# ========================================================================================================
+
+fcm_cluster <- function(distmat, m) {
+     cprime <- apply(distmat, 1L, function(dist_row) { sum( (1 / dist_row) ^ (2 / (m - 1)) ) })
+
+     u <- 1 / apply(distmat, 2L, function(dist_col) { cprime * dist_col ^ (2 / (m - 1)) })
+
+     u
+}
+
+# ========================================================================================================
+# Fuzzy objective function
+# ========================================================================================================
+
+fuzzy_objective <- function(u, distmat, m) {
+     sum(u^m * distmat^2)
+}
+
 # ========================================================================================================
 # Helper functions
 # ========================================================================================================
@@ -153,7 +173,7 @@ call_runminmax <- function(series, window) {
 
 # Create combinations of all possible pairs
 call_pairs <- function(n = 2L, lower = TRUE) {
-     if (n < 2)
+     if (n < 2L)
           stop("At least two elements are needed to create pairs.")
 
      .Call("pairs", n, lower, PACKAGE = "dtwclust")
@@ -164,14 +184,14 @@ check_parallel <- function() {
      if (is.null(foreach::getDoParName()))
           foreach::registerDoSEQ()
 
-     ret <- foreach::getDoParWorkers() > 1L
+     foreach::getDoParWorkers() > 1L
 }
 
 # Split a given object into tasks for parallel workers
 split_parallel <- function(obj, margin = NULL) {
      num_workers <- foreach::getDoParWorkers()
 
-     if (num_workers == 1)
+     if (num_workers == 1L)
           return(list(obj))
 
      if (is.null(margin))
@@ -183,21 +203,21 @@ split_parallel <- function(obj, margin = NULL) {
           stop("Invalid attempt to split an object into parallel tasks")
 
      num_tasks <- parallel::splitIndices(num_tasks, num_workers)
-     num_tasks <- num_tasks[sapply(num_tasks, length, USE.NAMES = FALSE) > 0]
+     num_tasks <- num_tasks[sapply(num_tasks, length, USE.NAMES = FALSE) > 0L]
 
      if (is.null(margin))
           ret <- lapply(num_tasks, function(id) obj[id])
      else
           ret <- switch(EXPR = margin,
                         lapply(num_tasks, function(id) obj[id, , drop = FALSE]),
-                        lapply(num_tasks, function(id) obj[ ,id, drop = FALSE])
+                        lapply(num_tasks, function(id) obj[ , id, drop = FALSE])
           )
 
      ret
 }
 
 # column-wise medians
-colMedians <- function(mat) { apply(mat, 2, stats::median) }
+colMedians <- function(mat) { apply(mat, 2L, stats::median) }
 
 # PREFUN for some of my proxy distances so that they support 'pairwise' direclty
 # Currently results in warnings because proxy does !is.na(reg_entry$PREFUN)
